@@ -50,8 +50,6 @@ class Reporter(object):
 
         for cu in self.code_units:
             try:
-                if not cu.relative:
-                    continue
                 statements, excluded, missing, _ = self.coverage._analyze(cu)
                 report_fn(cu, statements, excluded, missing)
             except KeyboardInterrupt:
@@ -59,4 +57,3 @@ class Reporter(object):
             except:
                 if not self.ignore_errors:
                     raise
-
